.packageName <- "genefilter"
#copyright 2001 R. Gentleman

#FILTER FUNCTIONS -- some trivial changes

allNA <- function(x) !all(is.na(x))

anyNA <- function(x) !any(is.na(x))

kOverA <- function(k, A=100, na.rm = TRUE) {
  function(x) {
      if(na.rm)
	x <- x[!is.na(x)]
      sum( x > A ) > k
  }
}

maxA <- function(A=75, na.rm=TRUE) {
    function(x) {max(x, na.rm=na.rm) >= A }
}


pOverA <-  function(p=0.05, A=100, na.rm = TRUE) {
  function(x) {
      if(na.rm)
	 x<-x[!is.na(x)]
      sum( x > A )/length(x) > p
  }
}


cv <- function(a=1, b=Inf, na.rm=TRUE) {
    function(x) {
        	sdx <- sd(x, na.rm=na.rm)
        if(is.na(sdx) || sdx == 0 ) return(FALSE)
	val <- sdx/abs(mean(x, na.rm=na.rm))
        if(val < a ) return(FALSE)
        if(val > b ) return(FALSE)
        return(TRUE)
            }
}

Anova <- function(cov, p=0.05, na.rm=TRUE)
{
    function(x) {
        if( na.rm ) {
            drop <- is.na(x)
            x <- x[!drop]
            cov <- cov[!drop]
        }
        m1 <- lm(x~cov)
        m2 <- lm(x~1)
        av <- anova(m2,m1)
        fstat <- av[["Pr(>F)"]][2]
        if( fstat < p )
            return(TRUE)
        return(FALSE)
    }
}

coxfilter <- function(surt, cens, p) {
   autoload("coxph", "survival")
   function(x) {
       srvd <- try(coxph(Surv(surt,cens)~x))
       if( inherits(srvd, "try-error") )
           return(FALSE)
       ltest <- -2*(srvd$loglik[1] - srvd$loglik[2])
       pv <- 1 - pchisq(ltest, 1)
       if( pv < p )
           return(TRUE)
       return(FALSE)
   }
}

ttest <- function(m, p=0.05, na.rm=TRUE) {
    if( length(m) == 1)
        function(x) {
            n <- length(x)
            if( m>n ) stop("m is larger than the number of samples")
            sub1 <- x[1:m]
            sub2 <- x[(m+1):n]
            if(na.rm) {
                drop <- is.na(x)
                sub1 <- sub1[!drop[1:m]]
                sub2 <- sub2[!drop[(m+1):n]]
            }
            t.test(sub1, sub2 )$p.value < p
        }
    else
        function(x) {
            if(na.rm) {
                drop <- is.na(x) | is.na(m)
                x<- x[!drop]
                m<- m[!drop]
            }
            t.test(x~m)$p.value < p
        }
  }


##a filter based on gaps

gapFilter <- function(Gap, IQR, Prop, na.rm=TRUE, neg.rm=TRUE) {
  function(x) {
     if(na.rm) x <- x[!is.na(x)]
     if(neg.rm) x <- x[x>0]
     lenx <- length(x)
     if( lenx < 4 || lenx < Prop+1 )
       return(FALSE)
     srtd <- sort(x)
     lq <- lenx*.25
     uq <- lenx*.75
     if( (srtd[uq] - srtd[lq]) > IQR )
        return(TRUE)
     if(Prop < 1)
        bot <- lenx*Prop
     else
        bot <- Prop
     top <- lenx - bot
     lag1 <- srtd[2:lenx]-srtd[1:(lenx-1)]
     if( max(lag1[bot:top]) > Gap )
       return(TRUE)
    return(FALSE)
  }
}


# Apply type functions


genefilter <- function(expr, flist)
    {
     if( inherits(expr, "exprSet") )
       expr <- exprs(expr)
     apply(expr, 1, flist)
}

filterfun <- function(...) {
     flist <- list(...)
 #let the user supply a list
     if( length(flist) == 1 && is.list(flist[[1]]) )
         flist <- flist[[1]]
     f <- function( x ) {
         for( fun in flist ) {
             fval <- fun(x)
             if( is.na(fval) || ! fval )
                 return(FALSE)
         }
             return(TRUE)
     }
     class(f) <- "filterfun"
     return(f)
 }





# genefinder.R
#
# genefinder functions.


genescale <- function (m, axis=2, method=c("Z", "R"), na.rm=TRUE) {
    ##scale by the range
    RscaleVector <- function(v, na.rm) {
        mm <- range(v, na.rm=na.rm)
        (v - mm[1]) / (mm[2] - mm[1])
    }
    ##scale using Zscore
    ZscaleVector <- function(v, na.rm)
        (v - mean(v, na.rm=na.rm))/sd(v, na.rm=na.rm)
#
# scales a matrix using the scaleVector function.
#
    which <- match.arg(method)
    method <- switch(which,
                     Z = ZscaleVector,
                     R = RscaleVector)
    if( is.matrix(m) || is.data.frame(m) ) {
        rval <- apply (m, axis, method, na.rm=na.rm)
        if( axis==1 ) return(t(rval))
        return(rval)
    }
    else
	method(m, na.rm=na.rm)
}

setGeneric("genefinder", function(X, ilist, numResults=25, scale="none",
    weights, method="euclidean" )
    standardGeneric("genefinder"))

setMethod("genefinder", c("exprSet", "vector", "ANY", "ANY", "ANY",
          "ANY"),
          function(X, ilist, numResults, scale, weights,
                   method) {
              gN <- geneNames(X)
              if (is.character(ilist))
                  ilist <- match(ilist,gN)
              ans <- genefinder(exprs(X), ilist, numResults, scale, weights,
                        method=method)
              names(ans) <- gN[ilist]
              ans
      })

setMethod("genefinder", c("matrix", "vector", "ANY", "ANY", "ANY"),
         function (X, ilist, numResults, scale, weights,
                        method=c("euclidean", "maximum", "manhattan",
                        "canberra", "correlation", "binary")) {
    X <- as.matrix(X)
    METHOD <- c("euclidean", "maximum", "manhattan",
                        "canberra", "correlation", "binary")
    method<-pmatch(method, METHOD)
    if (is.na(method))
        stop ("The distance method is invalid.")

    SCALE <- c("none", "range", "zscore")
    scale <- SCALE[pmatch(scale, SCALE)]

    # perform scaling if requested.
    #
    X <- switch(scale,
                none=X,
                range=genescale(X),
                zscore=scale(X),
                stop("The scaling method is invalid")
                )
    N <- nrow(X)
    C <- ncol(X)

    if( !is.vector(ilist) )
        stop("the genes to be compared to must be in a vector")

    ninterest <- length(ilist);

    if( is.character(ilist) ) {
        iRows <- match(ilist, row.names(X))
        names(iRows) <- ilist
    }
    else if ( is.numeric(ilist) ) {
        iRows <- ilist
        names(iRows) <- row.names(X)[ilist]
    }
    else
        stop("invalid genes selected")

    if( any(is.na(iRows)) )
        stop("invalid genes selected")

    if (missing(weights))
        weights <- rep(1,C)
    else if (length(weights) != C)
        stop("Supplied weights do not match number of columns")

    ## Do a sanity check on the requested genes in ilist -> if the
    ## gene exceeds the # of rows in the matrix, can not be processed.
    if (max(iRows) > N)
        stop("Requested genes exceed the dimensions of the supplied matrix.")


    Genes <- array(NA, dim=c(ninterest, numResults))
    Dists <- array(NA, dim=c(ninterest, numResults))
    extCall <- .C("mm_distance",
                  X = as.double(X),
                  nr= N,
                  nc= ncol(X),
                  g = as.integer(Genes),
                  d = as.double(Dists),
                  iRow  = as.integer(iRows),
                  nInterest = as.integer(ninterest),
                  nResults = as.integer(numResults),
                  method= as.integer(method),
                  weights = as.double(weights),
                  DUP = FALSE, NAOK=TRUE, PACKAGE="genefilter")

    Genes <- extCall$g+1
    Dists <- extCall$d
    Which <- vector()

    ## Get the number of genes/dists per selection.  There should
    ## always be a number of total genes such that they are a multiple
    ## of ninterest
    numPerList <- length(Genes) / ninterest

    Which <- rep(iRows, rep(numPerList, ninterest))

    byGene <- split(Genes, Which)
    names(byGene) <- rep("indices", length(byGene))
    byDists <- split(Dists, Which)
    names(byDists) <- rep("dists", length(byDists))
    ## Need a better way to stuff these together
    retList <- list()
    for (i in 1:ninterest) {
        retList[[i]] <- list(indices=byGene[[i]], dists=byDists[[i]])
    }

    return(retList)
})








.First.lib <- function(lib, pkgname, where) 
  {
  library.dynam("genefilter", pkgname, lib)
    if(missing(where)) {
        where <- match(paste("package:", pkgname, sep=""), search())
        if(is.na(where)) {
            warning(paste("Not a package name: ",pkgname))
            return()
        }
        where <- pos.to.env(where)
    }
  cacheMetaData(as.environment(where))
  }

