\name{Anova}
\alias{Anova}
\title{A filter function for Analysis of Variance }
\description{
  \code{Anova} returns a function of one argument with bindings for
  \code{cov} and \code{p}.
  The function, when evaluated, performs an ANOVA using \code{cov} as
  the covariate. It returns \code{TRUE} if the p value for a difference
  in means is less than \code{p}. 
}
\usage{
Anova(cov, p=0.05, na.rm=TRUE)
}
\arguments{
  \item{cov}{The covariate. It must have length equal to the number of
    columns of the array that \code{Anova} will be applied to. }
  \item{p}{ The p-value for the test. }
  \item{na.rm}{If set to \code{TRUE} any \code{NA}'s will be removed. }
}

\details{
  The function returned by \code{Anova} uses \code{lm} to fit a linear
  model of the form
  \code{lm(x ~ cov)}, where \code{x} is the set of gene expressions.
  The F statistic for an overall effect is computed and if it has a
  \emph{p}-value less than \code{p} the function returns \code{TRUE},
  otherwise it returns \code{FALSE} for that gene.
  }
\value{
 \code{Anova} returns a function with bindings for \code{cov} and
 \code{p} that will perform a one-way ANOVA.

  The covariate can be continuous, in which case the test is for a linear 
  effect for the covariate.
}
\author{R. Gentleman }
\seealso{\code{\link{kOverA}}, \code{\link{lm}} }

\examples{
  set.seed(123)
  af <- Anova(c(rep(1,5),rep(2,5)), .01)
  af(rnorm(10))
}
\keyword{manip}

\eof
\name{allNA}
\alias{allNA}
\alias{anyNA}
\title{ A filter function to determine if all elements of a vector are NA. }
\description{
  \code{allNA} evaluates to \code{FALSE} if all elements of its
  argument are \code{NA}. \code{anyNA} evaluates to \code{FALSE} if any
  of the elements of its argument are \code{NA}.
}
\usage{
allNA(x)
anyNA(x)
}

\arguments{
  \item{x}{The vector to test.}
}

\value{
  \code{FALSE} if all elements of \code{x} are \code{NA}.
}

\author{R. Gentleman}

\seealso{\code{\link{pOverA}}}

\examples{
  allNA(NA)
  allNA(1)
  anyNA(1)
  anyNA(NA)
}
\keyword{manip}

\eof
\name{coxfilter}
\alias{coxfilter}

\title{A filter function for univariate Cox regression. }
\description{
  A function that performs Cox regression with bindings for \code{surt},
  \code{cens}, and \code{p} is returned. This function filters genes
  according to the attained p-value from a Cox regression using
  \code{surt} as the survival times, and \code{cens} as the censoring
  indicator. It requires \code{survival}.
}
\usage{
coxfilter(surt, cens, p)
}
\arguments{
  \item{surt}{Survival times.}
  \item{cens}{Censoring indicator. }
  \item{p}{The p-value to use in filtering. }
}

\value{
  Calls to the \code{\link{coxph}} function in the \code{survival}
    library are used to fit a Cox model. The filter function returns
    \code{TRUE} if the p-value in the fit is less than \code{p}.
}

\author{R. Gentleman }
\seealso{\code{\link{Anova}}}

\examples{
   set.seed(-5)
   sfun <- coxfilter(rexp(10), ifelse(runif(10) < .7, 1, 0), .05)
   ffun <- filterfun(sfun)
   dat <- matrix(rnorm(1000), ncol=10)
   out <- genefilter(dat, ffun)
}
\keyword{manip}


\eof
\name{cv}
\alias{cv}
\title{A filter function for the coefficient of variation.}
\description{
  \code{cv} returns a function with values for \code{a} and \code{b}
  bound. This function takes a single argument. It computes the
  coefficient of variation for the input vector and returns \code{TRUE} if
  the coefficient of variation is between \code{a} and
  \code{b}. Otherwise it returns \code{FALSE}
}
\usage{
cv(a=1, b=Inf, na.rm=TRUE)
}
\arguments{
  \item{a}{The lower bound for the cv. }
  \item{b}{The upper bound for the cv. }
  \item{na.rm}{If set to \code{TRUE} any \code{NA}'s will be removed. }
}
\details{
  The coefficient of variation is the standard deviation divided by the 
  absolute value of the mean.
}
\value{
 It returns a function of one argument. The function has an environment
 with bindings for \code{a} and \code{b}.
}
\author{R. Gentleman }
\seealso{\code{\link{pOverA}}, \code{\link{kOverA}} }

\examples{
  set.seed(-3)
  cvfun <- cv(1,10)
  cvfun(rnorm(10,10))
  cvfun(rnorm(10))
}
\keyword{manip}


\eof
\name{filterfun}
\alias{filterfun}
\title{Creates a first FALSE exiting function from the list of
  filter functions it is given. }
\description{
  This function creates a function that takes a single argument. The
  filtering functions are bound in the environment of the returned
  function and are applied sequentially to the argument of the returned
  function. When the first filter function evaluates to \code{FALSE} the
  function returns \code{FALSE} otherwise it returns \code{TRUE}.
}
\usage{
filterfun(...)
}
\arguments{
  \item{...}{Filtering functions. }
}

\value{
 \code{filterfun} returns a function that takes a single argument. It
 binds the filter functions given to it in the environment of the
 returned function. These functions are applied sequentially (in the
 order they were given to \code{filterfun}). The function returns
 \code{FALSE} (and exits) when the first filter function returns
 \code{FALSE} otherwise it returns \code{TRUE}.
}
\author{R. Gentleman }
\seealso{\code{\link{genefilter}} }
\examples{
 set.seed(333)
 x <- matrix(rnorm(100,2,1),nc=10)
 cvfun <- cv(.5,2.5)
 ffun <- filterfun(cvfun)
 which <- genefilter(x, ffun)
}
\keyword{manip}


\eof
\name{gapFilter}
\alias{gapFilter}
\title{ A filter to select genes based on there being a gap. }
\description{
  The \code{gapFilter} looks for genes that might usefully discriminate
  between two groups (possibly unknown at the time of filtering).
  To do this we look for a gap in the ordered expression values. The gap
  must come in the central portion (we exclude jumps in the initial
  \code{Prop} values or the final \code{Prop} values).
  Alternatively, if the IQR for the gene is large that will also pass
  our test and the gene will be selected.
}
\usage{
gapFilter(Gap, IQR, Prop, na.rm=TRUE, neg.rm=TRUE)
}
\arguments{
  \item{Gap}{The size of the gap required to pass the test. }
  \item{IQR}{The size of the IQR required to pass the test. }
  \item{Prop}{The proportion (or number) of samples to exclude at either
  end.}
  \item{na.rm}{If \code{TRUE} then \code{NA}'s will be removed before
    processing. }
  \item{neg.rm}{ If \code{TRUE} then negative values in \code{x} will be
    removed before processing.}
}
\details{
  As stated above we are interested in 
}
\value{
 A function that returns either \code{TRUE} or \code{FALSE} depending on
 whether the vector supplied has a gap larger than \code{Gap} or an IQR
 (inter quartile range) larger than \code{IQR}. For computing the gap we
 want to exclude a proportion, \code{Prop} from either end of the sorted
 values. The reason for this requirement is that genes which differ in
 expression levels only for a few samples are not likely to be interesting.
}

\author{R. Gentleman }

\seealso{\code{\link{ttest}}, \code{\link{genefilter}} }

\examples{
 set.seed(256)
 x <- c(rnorm(10,100,3), rnorm(10, 100, 10))
 y <- x + c(rep(0,10), rep(100,10))
 tmp <- rbind(x,y) 
 Gfilter <- gapFilter(200, 100, 5)
 ffun <- filterfun(Gfilter)
 genefilter(tmp, ffun)
}
\keyword{manip}

\eof
\name{genefilter}
\alias{genefilter}

\title{A function to filter genes. }
\description{
   \code{genefilter} filters genes in the array \code{expr} using the
   filter functions in \code{flist}. It returns an array of logical
   values (suitable for subscripting) of the same length as there are
   rows in \code{expr}. For each row of \code{expr} the returned value
   is \code{TRUE} if the row passed all the filter functions. Otherwise
   it is set to \code{FALSE}.
}
\usage{
genefilter(expr, flist)
}
\arguments{
  \item{expr}{A matrix or \code{exprSet} that the filter functions will be
    applied to. }
  \item{flist}{A list of filter functions to apply to the array. }
}
\details{
   This package uses a very simple but powerful protocol for
   \emph{filtering} genes. The user simply constructs any number of
   tests that they want to apply. A test is simply a function (as
   constructed using one of the many helper functions in this package)
   that returns \code{TRUE} if the gene of interest passes the test (or
   filter) and \code{FALSE} if the gene of interest fails.

   The benefit of this approach is that each test is constructed
   individually (and can be tested individually). The tests are then
   applied sequentially to each gene. The function returns a logical
   vector indicating whether the gene passed all tests functions or
   failed at least one of them.
   
  }
\value{
  A logical vector of length equal to the number of rows of \code{expr}.
  The values in that vector indicate whether the corresponding row of
  \code{expr} passed the set of filter functions.
}

\author{R. Gentleman }
\seealso{\code{\link{genefilter}}, \code{\link{kOverA}} }

\examples{
   set.seed(-1)
   f1 <- kOverA(5, 10)
   flist <- filterfun(f1, allNA)
   exprA <- matrix(rnorm(1000, 10), nc=10)
   ans <- genefilter(exprA, flist)
}
\keyword{manip}


\eof
\name{genefinder}
\alias{genefinder}
\alias{genefinder,exprSet,vector-method}
\alias{genefinder,matrix,vector-method}
\title{ Finds genes that have similar patterns of expression. }
\description{
  Given an \code{exprSet} or a matrix of gene expressions, and the
  indices of the genes of 
  interest, \code{genefinder} returns a list of the
  \code{numbResults} closest genes.
  The user can specify one of the standard distance measures listed
  below.
  The number of values to return can be specified. The return value is a
  list with two components:
  genes (measured through the desired distance method) to the genes
  of interest (where X is the number of desired results returned) and
  their distances.
}
\usage{
genefinder(X, ilist, numResults=25, scale="none", weights, method="euclidean")
}
\arguments{
  \item{X}{ A numeric matrix where columns represent patients, and rows
    represent genes. }
  \item{ilist}{ Vector of genes of interest. Contains indices of genes
    in matrix X. }
  \item{numResults}{Number of results to display, starting from the least
    distance to the greatest. }
  \item{scale}{ one of 'none', 'range', or 'zscore'. Scaling
    is carried out separately on each row.}
  \item{weights}{A vector of weights applied across the columns of
    \code{X}.  If no weights are supplied, no weights are applied}.
  \item{method}{ one of "euclidean", "maximum", "manhattan", "canberra", 
        "correlation", "binary".  }
}
\details{
  If the scale option is "range", then the input matrix is scaled using
  genescale(). If it is "zscore", then the input matrix is scaled using
  the 'scale' builtin with no arguments.

  The method option specifies the metric used for gene comparisons. The
  metric is applied, row by row, for each gene specified in \code{ilist}.

  The "correlation" option for the distance method will return a value
  equal to 1-correlation(x).

  See \code{\link{dist}} for a more detailed description of the distances.
}
\value{
  The returned value is a list containing an entry for each gene
  specified in ilist.  Each list entry contains an array of distances
  for that gene of interest.
}
\author{ J. Gentry and M. Kajen }

\seealso{\code{\link{genescale}} }

\examples{
set.seed(12345)
#create some fake expression profiles
m1 <- matrix (1:12, 4, 3)
v1 <- 1
nr <- 2
#find the 2 rows of m1 that are closest to row 1
genefinder (m1, v1, nr, method="euc")


v2 <- c(1,3)
genefinder (m1, v2, nr)

genefinder (m1, v2, nr, scale="range")

genefinder (m1, v2, nr, method="manhattan")

m2 <- matrix (rnorm(100), 10, 10)
v3 <- c(2, 5, 6, 8)
nr2 <- 6
genefinder (m2, v3, nr2, scale="zscore")

\testonly{
	m1 <- matrix(rnorm(1000),100,10)
	v1 <- c(3,5,8,42)
	nr2 <- 35
	genefinder(m1,v1,nr2,method="euclidean")
	genefinder(m1,v1,nr2,method="maximum")
	genefinder(m1,v1,nr2,method="canberra")
	genefinder(m1,v1,nr2,method="binary")
	genefinder(m1,v1,nr2,method="correlation")
	
	m2 <- matrix(rnorm(10000),1000,10)
	v1 <- c(1,100,563,872,921,3,52,95,235,333)
	nr <- 100
	genefinder(m2,v1,nr2,scale="zscore",method="euclidean")
	genefinder(m2,v1,nr2,scale="range", method="maximum")
	genefinder(m2,v1,nr2,scale="zscore", method="canberra")
	genefinder(m2,v1,nr2,scale="range", method="binary")
	genefinder(m2,v1,nr2,scale="zscore",method="correlation")
	}

}
\keyword{ manip }


\eof
\name{genescale}
\alias{genescale}
\title{Scales a matrix or vector.}
\description{
  \code{genescale} returns a scaled version of the input matrix m by applying
  the following formula to each column of the matrix:
     {    y[i] = ( x[i] - min(x) ) / ( max(x) - min(x) ) }
}
\usage{
genescale(m, axis=2, method=c("Z", "R"), na.rm=TRUE)
}

\arguments{
  \item{m}{Input a matrix or a vector with numeric elements. }
  \item{axis}{An integer indicating which axis of \code{m} to scale.}
  \item{method}{Either "Z" or "R", indicating whether a Z scaling or a
    range scaling should be performed.}
  \item{na.rm}{A boolean indicating whether \code{NA}'s should be
    removed.}
}
\details{
  Either the rows or columns of \code{m} are scaled. This is done either
  by subtracting the mean and dividing by the standard deviation ("Z")
  or by subtracing the minimum and dividing by the range.
}
\value{
  A scaled version of the input.
  If \code{m} is a \code{matrix} or a \code{dataframe} then the
  dimensions of the returned value agree with that of \code{m},
  in both cases the returned value is a \code{matrix}.
}

\author{ R. Gentleman }

\seealso{ \code{\link{genefinder}},\code{\link{scale}} }

\examples{
  m <- matrix(1:12, 4, 3)
  genescale(m)
}
\keyword{ manip }


\eof
\name{kOverA}
\alias{kOverA}
\title{A filter function for k elements larger than A. }
\description{
  \code{kOverA} returns a filter function with bindings for \code{k} and
  \code{A}. This function evaluates to \code{TRUE} if at least \code{k}
  of the arguments elements are larger than \code{A}.
}
\usage{
kOverA(k, A=100, na.rm=TRUE)
}
\arguments{
  \item{A}{The value you want to exceed. }
  \item{k}{The number of elements that have to exceed A.}
  \item{na.rm}{If set to \code{TRUE} any \code{NA}'s will be removed. }
}

\value{
 A function with bindings for \code{A} and \code{k}.
}

\author{R. Gentleman}

\seealso{\code{\link{pOverA}}}

\examples{
   fg <- kOverA(5, 100)
   fg(90:100)
   fg(98:110)
}
\keyword{manip}

\eof
\name{maxA}
\alias{maxA}
\title{ A filter function to filter according to the maximum. }
\description{
  \code{maxA} returns a function with the parameter \code{A} bound.
  The returned function evaluates to \code{TRUE} if any element of its
  argument is larger than \code{A}.
}
\usage{
maxA(A=75, na.rm=TRUE)
}
\arguments{
  \item{A}{The value that at least one element must exceed. }
  \item{na.rm}{If \code{TRUE} then \code{NA}'s are removed. }
}

\value{
 \code{maxA} returns a function with an environment containing a binding
 for \code{A}.
 
}

\author{R. Gentleman }

\seealso{\code{\link{pOverA}} }

\examples{
   ff <- maxA(30)
   ff(1:10)
   ff(28:31)
}
\keyword{manip}


\eof
\name{pOverA}
\alias{pOverA}

\title{A filter function to filter according to the proportion of
  elements larger than A. }
\description{
  A function that returns a function with values for \code{A}, \code{p}
  and \code{na.rm} bound to the specified values. The function takes a
  single vector, \code{x}, as an argument.
  When the returned function is evaluated it returns \code{TRUE} if the
  proportion of values in \code{x} that are larger than \code{A} exceeds
  \code{p}. 
}
\usage{
pOverA(p=0.05, A=100, na.rm=TRUE)
}
\arguments{
  \item{A}{The value to be exceeded. }
  \item{p}{The proportion that need to exceed \code{A} for \code{TRUE}
    to be returned. }
  \item{na.rm}{ If \code{TRUE} then \code{NA}'s are removed. }
}

\value{
 \code{pOverA} returns a function with bindings for \code{A}, \code{p}
 and \code{na.rm}. This function evaluates to \code{TRUE} if the
  proportion of values in \code{x} that are larger than \code{A} exceeds
  \code{p}. 
}
\author{R. Gentleman}

\seealso{  \code{\link{cv}} }

\examples{
  ff<- pOverA(p=.1, 10)
  ff(1:20)
  ff(1:5)
}
\keyword{manip}

\eof
\name{tdata}
\alias{tdata}
\non_function{}
\title{A small test dataset of Affymetrix Expression data. }
\usage{data(tdata)}
\description{
The \code{tdata} data frame has 500 rows and 26 columns.
The columns correspond to samples while the rows correspond to genes.
The row names are Affymetrix accession numbers.
}
\format{
  This data frame contains 26 columns.
}

\source{
 An unknown data set.
}

\examples{
data(tdata)
}
\keyword{datasets}

\eof
\name{ttest}
\alias{ttest}
\title{A filter function for a t.test }
\description{
  \code{ttest} returns a function of one argument with bindings for
  \code{cov} and \code{p}.
  The function, when evaluated, performs a t-test using \code{cov} as
  the covariate. It returns \code{TRUE} if the p value for a difference
  in means is less than \code{p}. 
}
\usage{
ttest(m, p=0.05, na.rm=TRUE)
}
\arguments{
  \item{m}{If \code{m} is of length one then it is assumed that elements
  one through \code{m} of \code{x} will be one group. Otherwise \code{m}
  is presumed to be the same length as \code{x} and constitutes the
  groups.}
  \item{p}{ The p-value for the test. }
  \item{na.rm}{If set to \code{TRUE} any \code{NA}'s will be removed. }
}
\details{
   When the data can be split into two groups (diseased and normal for
   example) then we often want to select genes on their ability to
   distinguish those two groups. The t-test is well suited to this and
   can be used as a filter function.

   This helper function creates a t-test (function) for the specified
   covariate and considers a gene to have passed the filter if the
   p-value for the gene is less than the prespecified \code{p}.
  }
\value{
 \code{ttest} returns a function with bindings for \code{m} and
 \code{p} that will perform a t-test}.
}
\author{R. Gentleman }
\seealso{\code{\link{kOverA}}, \code{\link{Anova}}, \code{\link{t.test}} }

\examples{
  dat <- c(rep(1,5),rep(2,5))
  set.seed(5)
  y <- rnorm(10)
  af <- ttest(dat, .01)
  af(y)
  af2 <- ttest(5, .01)
  af2(y)
  y[8] <- NA
  af(y)
  af2(y)
  y[1:5] <- y[1:5]+10
  af(y)
}
\keyword{manip}

\eof
